@@ -1,6 +1,7 @@
 //! Serialize a Rust data structure into AT Command strings
 
 use core::fmt::{self, Write};
+use core::ops::Deref;
 
 use serde::ser;
 
@@ -37,16 +38,24 @@ pub type Result<T> = ::core::result::Result<T, Error>;
 /// };
 /// let s: String<32> = to_string(
 ///     &b,
-///     String::<32>::from("+CMD"),
+///     "+CMD",
 ///     SerializeOptions::default(),
 /// )
 /// .unwrap();
 ///
 /// assert_eq!(s, String::<32>::from("AT+CMD=Some bytes\r\n"));
 /// ```
-#[derive(Clone, PartialEq)]
+#[derive(Debug, Clone, PartialEq)]
 pub struct Bytes<'a>(pub &'a [u8]);
 
+impl Deref for Bytes<'_> {
+    type Target = [u8];
+
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
+
 impl<'a> serde::Serialize for Bytes<'a> {
     fn serialize<S>(&self, serializer: S) -> ::core::result::Result<S::Ok, S::Error>
     where
@@ -125,14 +134,14 @@ impl fmt::Display for Error {
 #[cfg(any(test, feature = "std"))]
 impl std::error::Error for Error {}
 
-pub(crate) struct Serializer<'a, const B: usize, const C: usize> {
+pub(crate) struct Serializer<'a, const B: usize> {
     buf: Vec<u8, B>,
-    cmd: String<C>,
+    cmd: &'a str,
     options: SerializeOptions<'a>,
 }
 
-impl<'a, const B: usize, const C: usize> Serializer<'a, B, C> {
-    fn new(cmd: String<C>, options: SerializeOptions<'a>) -> Self {
+impl<'a, const B: usize> Serializer<'a, B> {
+    fn new(cmd: &'a str, options: SerializeOptions<'a>) -> Self {
         Serializer {
             buf: Vec::new(),
             cmd,
@@ -141,6 +150,20 @@ impl<'a, const B: usize, const C: usize> Serializer<'a, B, C> {
     }
 }
 
+/// Upper-case hex for value in 0..16, encoded as ASCII bytes
+fn hex_4bit(c: u8) -> u8 {
+    if c <= 9 {
+        0x30 + c
+    } else {
+        0x41 + (c - 10)
+    }
+}
+
+/// Upper-case hex for value in 0..256, encoded as ASCII bytes
+fn hex(c: u8) -> (u8, u8) {
+    (hex_4bit(c >> 4), hex_4bit(c & 0x0F))
+}
+
 // NOTE(serialize_*signed) This is basically the numtoa implementation minus the lookup tables,
 // which take 200+ bytes of ROM / Flash
 macro_rules! serialize_unsigned {
@@ -207,16 +230,16 @@ macro_rules! serialize_fmt {
     }};
 }
 
-impl<'a, 'b, const B: usize, const C: usize> ser::Serializer for &'a mut Serializer<'b, B, C> {
+impl<'a, 'b, const B: usize> ser::Serializer for &'a mut Serializer<'b, B> {
     type Ok = ();
     type Error = Error;
     type SerializeSeq = Unreachable;
     type SerializeTuple = Unreachable;
     type SerializeTupleStruct = Unreachable;
-    type SerializeTupleVariant = SerializeTupleVariant<'a, 'b, B, C>;
+    type SerializeTupleVariant = SerializeTupleVariant<'a, 'b, B>;
     type SerializeMap = Unreachable;
-    type SerializeStruct = SerializeStruct<'a, 'b, B, C>;
-    type SerializeStructVariant = SerializeStructVariant<'a, 'b, B, C>;
+    type SerializeStruct = SerializeStruct<'a, 'b, B>;
+    type SerializeStructVariant = SerializeStructVariant<'a, 'b, B>;
 
     fn serialize_bool(self, v: bool) -> Result<Self::Ok> {
         if v {
@@ -285,7 +308,65 @@ impl<'a, 'b, const B: usize, const C: usize> ser::Serializer for &'a mut Seriali
 
     fn serialize_str(self, v: &str) -> Result<Self::Ok> {
         self.buf.push(b'"')?;
-        self.buf.extend_from_slice(v.as_bytes())?;
+
+        // Do escaping according to "6. MUST represent all strings (including object member names) in
+        // their minimal-length UTF-8 encoding": https://gibson042.github.io/canonicaljson-spec/
+        //
+        // We don't need to escape lone surrogates because surrogate pairs do not exist in valid UTF-8,
+        // even if they can exist in JSON or JavaScript strings (UCS-2 based). As a result, lone surrogates
+        // cannot exist in a Rust String. If they do, the bug is in the String constructor.
+        // An excellent explanation is available at https://www.youtube.com/watch?v=HhIEDWmQS3w
+
+        // Temporary storage for encoded a single char.
+        // A char is up to 4 bytes long wehn encoded to UTF-8.
+        let mut encoding_tmp = [0u8; 4];
+
+        for c in v.chars() {
+            match c {
+                '\\' => {
+                    self.buf.push(b'\\')?;
+                    self.buf.push(b'\\')?;
+                }
+                '"' => {
+                    self.buf.push(b'\\')?;
+                    self.buf.push(b'"')?;
+                }
+                '\u{0008}' => {
+                    self.buf.push(b'\\')?;
+                    self.buf.push(b'b')?;
+                }
+                '\u{0009}' => {
+                    self.buf.push(b'\\')?;
+                    self.buf.push(b't')?;
+                }
+                '\u{000A}' => {
+                    self.buf.push(b'\\')?;
+                    self.buf.push(b'n')?;
+                }
+                '\u{000C}' => {
+                    self.buf.push(b'\\')?;
+                    self.buf.push(b'f')?;
+                }
+                '\u{000D}' => {
+                    self.buf.push(b'\\')?;
+                    self.buf.push(b'r')?;
+                }
+                '\u{0000}'..='\u{001F}' => {
+                    self.buf.push(b'\\')?;
+                    self.buf.push(b'u')?;
+                    self.buf.push(b'0')?;
+                    self.buf.push(b'0')?;
+                    let (hex1, hex2) = hex(c as u8);
+                    self.buf.push(hex1)?;
+                    self.buf.push(hex2)?;
+                }
+                _ => {
+                    let encoded = c.encode_utf8(&mut encoding_tmp as &mut [u8]);
+                    self.buf.extend_from_slice(encoded.as_bytes())?;
+                }
+            }
+        }
+
         self.buf.push(b'"')?;
         Ok(())
     }
@@ -408,15 +489,15 @@ impl<'a, 'b, const B: usize, const C: usize> ser::Serializer for &'a mut Seriali
 }
 
 /// Serializes the given data structure as a string
-pub fn to_string<T, const B: usize, const C: usize>(
+pub fn to_string<T, const B: usize>(
     value: &T,
-    cmd: String<C>,
+    cmd: &str,
     options: SerializeOptions<'_>,
 ) -> Result<String<B>>
 where
     T: ser::Serialize + ?Sized,
 {
-    let mut ser = Serializer::<B, C>::new(cmd, options);
+    let mut ser = Serializer::<B>::new(cmd, options);
     value.serialize(&mut ser)?;
     Ok(String::from(unsafe {
         core::str::from_utf8_unchecked(&ser.buf)
@@ -424,9 +505,9 @@ where
 }
 
 /// Serializes the given data structure as a byte vector
-pub fn to_vec<T, const B: usize, const C: usize>(
+pub fn to_vec<T, const B: usize>(
     value: &T,
-    cmd: String<C>,
+    cmd: &str,
     options: SerializeOptions<'_>,
 ) -> Result<Vec<u8, B>>
 where
@@ -562,7 +643,7 @@ mod tests {
     fn tuple_struct() {
         let s: String<32> = to_string(
             &PacketSwitchedParam::QoSDelay3G(15),
-            String::<32>::from(""),
+            "",
             SerializeOptions::default(),
         )
         .unwrap();
@@ -572,12 +653,7 @@ mod tests {
 
     #[test]
     fn newtype_struct() {
-        let s: String<32> = to_string(
-            &Handle(15),
-            String::<32>::from(""),
-            SerializeOptions::default(),
-        )
-        .unwrap();
+        let s: String<32> = to_string(&Handle(15), "", SerializeOptions::default()).unwrap();
 
         assert_eq!(s, String::<32>::from("15"));
     }
@@ -592,8 +668,7 @@ mod tests {
         let b = WithBytes {
             s: Bytes(&slice[..]),
         };
-        let s: String<32> =
-            to_string(&b, String::<32>::from("+CMD"), SerializeOptions::default()).unwrap();
+        let s: String<32> = to_string(&b, "+CMD", SerializeOptions::default()).unwrap();
         assert_eq!(s, String::<32>::from("AT+CMD=Some bytes\r\n"));
     }
 
@@ -602,8 +677,7 @@ mod tests {
         let test: CharVec<8> =
             CharVec(heapless::Vec::from_slice(&['I', 'M', 'P', '_', 'M', 'S', 'G']).unwrap());
 
-        let s: String<32> =
-            to_string(&test, String::<32>::from(""), SerializeOptions::default()).unwrap();
+        let s: String<32> = to_string(&test, "", SerializeOptions::default()).unwrap();
         assert_eq!(s, String::<32>::from("IMP_MSG"));
 
         #[derive(Clone, PartialEq, Serialize)]
@@ -615,8 +689,7 @@ mod tests {
             s: CharVec(heapless::Vec::from_slice(&['I', 'M', 'P', '_', 'M', 'S', 'G']).unwrap()),
             n: 12,
         };
-        let s: String<32> =
-            to_string(&b, String::<32>::from("+CMD"), SerializeOptions::default()).unwrap();
+        let s: String<32> = to_string(&b, "+CMD", SerializeOptions::default()).unwrap();
         assert_eq!(s, String::<32>::from("AT+CMD=IMP_MSG,12\r\n"));
     }
 }
